// $Id: dockapp.cc,v 1.15 2003/02/09 19:28:00 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

#include "dockapp.h"

//--------------------------------------------------------------------
DockApp::DockApp(Mixer* mixer, int argc, char** argv,
		 const string& display, const string& position, 
		 const string& theme_file, char x_flags)
  : mixer_(mixer), icons_(NULL)
{
  graphics_ = new Graphics(argc, argv, display, position, x_flags);

  try
  {
    theme_ = new Theme(theme_file);
    theme_->initImages(getXHandler());
    icons_ = new unsigned[theme_->icon_num_];
  }
  catch(InvalidThemeException& e)
  {
    std::cerr << e.getErrorMessage() << std::endl;
    exit(1);
  }
#ifdef __USE_ALSA__
  for(unsigned count = 0; count < mixer->getNumChannels(); count++)
  {
    if(count > theme_->icon_num_)
      icons_[count] = theme_->icon_num_ - 1;
    else
    {
      if(mixer->getChannelName(count) == "Master")
	icons_[count] = 0;
      else if(mixer->getChannelName(count) == "PCM")
	icons_[count] = 4;
      else if(mixer->getChannelName(count),"MIC")
	icons_[count] = 7;
      else if(mixer->getChannelName(count),"Line")
	icons_[count] = 6;
      else if(mixer->getChannelName(count),"CD")
	icons_[count] = 8;
      else if(mixer->getChannelName(count),"Synth")
	icons_[count] = 3;
      else if(mixer->getChannelName(count),"PC Speaker")
	icons_[count] = 5;
      else if(mixer->getChannelName(count),"Bass")
	icons_[count] = 1;
      else if(mixer->getChannelName(count),"Treble")
	icons_[count] = 2;
      else
	icons_[count] = theme_->icon_num_ - 1;
    }
  }
  
#else
  for(unsigned count = 0; count < mixer->getNumChannels(); count++)
  {
    if(count > theme_->icon_num_)
      icons_[count] = theme_->icon_num_ - 1;
    else
      icons_[count] = count;
  }
#endif

}

//--------------------------------------------------------------------
DockApp::~DockApp()
{
  delete graphics_;
  delete theme_;
  delete icons_;
}

//--------------------------------------------------------------------
void DockApp::drawBar(unsigned value, unsigned offset, unsigned width)
{
  Color *current_color = NULL;
  unsigned pos;

  for(unsigned count = 0; count < theme_->bar_num_; count++)
  {
    pos = (value * theme_->bar_num_) / 100;

    if(count >= pos)
    {
      current_color = theme_->bar_color_inactive_;
    }
    else
    {
      current_color = theme_->bar_colors_[count];
    }

    if(theme_->bar_alignment_ == Theme::VERTICAL)
    {
      graphics_->drawRectangle(current_color, 
			       theme_->bar_x_ + offset,
			       theme_->bar_y2_ - count *
			       (theme_->bar_thickness_ + theme_->bar_gap_),
			       width, 
			       theme_->bar_thickness_);
    } 
    else
    {
      graphics_->drawRectangle(current_color, 
			       theme_->bar_x_ + theme_->bar_height_
			       - theme_->bar_height_ * pos, 
			       theme_->bar_y_ + offset,
			       theme_->bar_thickness_, 
			       width);
    }
  }
}

//--------------------------------------------------------------------
void DockApp::drawLeftBar(unsigned value)
{
  drawBar(value, 0, (theme_->bar_width_ >> 1) -1);
}

//--------------------------------------------------------------------
void DockApp::drawRightBar(unsigned value)
{
  drawBar(value, (theme_->bar_width_ >> 1) + 1, (theme_->bar_width_ >> 1) -1);
}

//--------------------------------------------------------------------
void DockApp::drawMonoBar(unsigned value)
{
  drawBar(value, 0, theme_->bar_width_);
}

//--------------------------------------------------------------------
XHandler* DockApp::getXHandler() const
{
  return graphics_->getXHandler();
}

//--------------------------------------------------------------------
Display*  DockApp::getDisplay() const
{
  return graphics_->getXHandler()->getDisplay();
}

//--------------------------------------------------------------------
void DockApp::drawButtons(bool state_left, bool state_right,
			  bool state_mute, bool state_rec,
			  unsigned current_channel)
{
  drawLeftButton(state_left);
  drawRightButton(state_right);
  drawMuteButton(state_mute);
  drawRecButton(state_rec, current_channel);
}

//--------------------------------------------------------------------
void DockApp::drawLeftButton(bool state)
{
  graphics_->drawButton(theme_->button_left_image_,
			theme_->button_left_x_,
			theme_->button_left_y_,
			theme_->button_left_width_,
			theme_->button_left_height_,
			state);
}

//--------------------------------------------------------------------
void DockApp::drawRightButton(bool state)
{
  graphics_->drawButton(theme_->button_right_image_,
			theme_->button_right_x_,
			theme_->button_right_y_,
			theme_->button_right_width_,
			theme_->button_right_height_,
			state);
}

//--------------------------------------------------------------------
void DockApp::drawMuteButton(bool state)
{
  graphics_->drawButton(theme_->button_mute_image_,
			theme_->button_mute_x_,
			theme_->button_mute_y_,
			theme_->button_mute_width_,
			theme_->button_mute_height_,
			state);
}

//--------------------------------------------------------------------
void DockApp::drawRecButton(bool state, unsigned current_channel)
{
  //  if(mixer_->canCapture(current_channel))
  if(mixer_->canCapture(current_channel))
  {
    graphics_->drawButton(theme_->button_rec_image_,
			  theme_->button_rec_x_,
			  theme_->button_rec_y_,
			  theme_->button_rec_width_,
			  theme_->button_rec_height_,
			  state);
  }
  else
  {
    graphics_->drawButton(theme_->button_rec_image_,
			  theme_->button_rec_x_,
			  theme_->button_rec_y_,
			  theme_->button_rec_width_,
			  theme_->button_rec_height_,
			  false);
  }
}

//--------------------------------------------------------------------
void DockApp::drawIcon(unsigned icon)
{
  // TODO: Special icon for channel without icon?
  if(icon < theme_->icon_num_)
  {
    graphics_->drawImageOffset(theme_->icon_images_,
			       icons_[icon] * theme_->icon_width_,
			       0,
			       theme_->icon_x_,
			       theme_->icon_y_,
			       theme_->icon_width_,
			       theme_->icon_height_);
  } 
  else
  {
    graphics_->drawImageOffset(theme_->icon_images_,
			       theme_->icon_num_ * theme_->icon_width_ ,
			       0,
			       theme_->icon_x_,
			       theme_->icon_y_,
			       theme_->icon_width_,
			       theme_->icon_height_);
  }
}

//--------------------------------------------------------------------
void DockApp::drawLabel(const string& label)
{
  graphics_->drawText(label.c_str(),
		      theme_->label_font_,
		      theme_->label_x_,
		      theme_->label_y_,
		      theme_->label_width_,
		      theme_->label_height_);
}

//--------------------------------------------------------------------
void DockApp::drawCounter(unsigned value)
{
  char text[4];
  if(value>100) return;

  sprintf(text, "%d", value);
  graphics_->drawText(text,
		      theme_->label_font_,
		      theme_->label_x_,
		      theme_->label_y_,
		      theme_->label_width_,
		      theme_->label_height_);
}

//--------------------------------------------------------------------
void DockApp::drawBackground()
{
  // FIXXXME: width and height of dockapp!!
    graphics_->drawImage(theme_->background_image_, 0, 0,
     		       64, 64);
}


//--------------------------------------------------------------------
void DockApp::update(unsigned current_channel, bool state_left, 
		     bool state_right, bool state_mute,
		     bool state_rec, unsigned dragging)
{
  drawBackground();

  if(mixer_->isMono(current_channel))
  {
    drawMonoBar(mixer_->getChannelLeft(current_channel));
  }
  else
  {
    drawRightBar(mixer_->getChannelRight(current_channel));
    drawLeftBar(mixer_->getChannelLeft(current_channel));
  }
  
  drawButtons(state_left, state_right, state_mute, state_rec, current_channel);

  drawIcon(current_channel);

  if(dragging)
  {
    drawCounter(dragging);
  }
  else
    drawLabel(mixer_->getChannelName(current_channel));

  graphics_->redraw();
}


//--------------------------------------------------------------------
bool DockApp::isLeftButton(unsigned x, unsigned y) const
{
  return (x >= theme_->button_left_x_ &&
	  x <= theme_->button_left_x_ + theme_->button_left_width_ &&
	  y >= theme_->button_left_y_ &&
	  y <= theme_->button_left_y_ + theme_->button_left_height_);
	  
}

//--------------------------------------------------------------------
bool DockApp::isRightButton(unsigned x, unsigned y) const
{
  return (x >= theme_->button_right_x_ &&
	  x <= theme_->button_right_x_ + theme_->button_right_width_ &&
	  y >= theme_->button_right_y_ &&
	  y <= theme_->button_right_y_ + theme_->button_right_height_);
	  
}

//--------------------------------------------------------------------
bool DockApp::isMuteButton(unsigned x, unsigned y) const
{
  return (x >= theme_->button_mute_x_ &&
	  x <= theme_->button_mute_x_ + theme_->button_mute_width_ &&
	  y >= theme_->button_mute_y_ &&
	  y <= theme_->button_mute_y_ + theme_->button_mute_height_);
	  
}

//--------------------------------------------------------------------
bool DockApp::isRecButton(unsigned x, unsigned y) const
{
  return (x >= theme_->button_rec_x_ &&
	  x <= theme_->button_rec_x_ + theme_->button_rec_width_ &&
	  y >= theme_->button_rec_y_ &&
	  y <= theme_->button_rec_y_ + theme_->button_rec_height_);
	  
}

//--------------------------------------------------------------------
bool DockApp::isVolumeBar(unsigned x, unsigned y) const
{
  return (x >= theme_->bar_x_ &&
	  x <= theme_->bar_x_ + theme_->bar_width_ &&
	  y >= theme_->bar_y_ &&
	  y <= theme_->bar_y_ + theme_->bar_height_);
	  
}

//--------------------------------------------------------------------
bool DockApp::isLeftVolumeBar(unsigned x, unsigned y) const
{
  if(theme_->bar_alignment_ == Theme::VERTICAL)
  {
    return x < theme_->bar_x_ + (theme_->bar_width_ >> 1) + 5;
  } 
  else
  {
    return y < theme_->bar_y_ + (theme_->bar_height_ >> 1) + 5;
  }
}

//--------------------------------------------------------------------
bool DockApp::isRightVolumeBar(unsigned x, unsigned y) const
{
  if(theme_->bar_alignment_ == Theme::VERTICAL)
  {
    return x > theme_->bar_x_ + (theme_->bar_width_ >> 1) - 5;
  } 
  else
  {
    return y > theme_->bar_y_ + (theme_->bar_height_ >> 1) - 5;
  }
}

//--------------------------------------------------------------------
unsigned DockApp::normalize(unsigned x, unsigned y) const
{
  if(theme_->bar_alignment_ == Theme::VERTICAL)
  {
    return 2 + (100 * (theme_->bar_height_ - y + theme_->bar_y_) / theme_->bar_height_);
  } 
  else
  {
    return 2 + (100 * (theme_->bar_width_ - x + theme_->bar_x_) / theme_->bar_width_);
  }  
}
