// $Id: oss_mixer.cc,v 1.16 2003/03/11 19:40:42 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
// Parts copyright (C) 1998  Sam Hawker <shawkie@geocities.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

#include "oss_mixer.h"


//----------------------------------------------------------------------
OSSMixer::OSSMixer()
  throw(MixerDeviceException)
  : Mixer(DEFAULT_DEVICE), modify_counter_(-1)
{
  try
  {
    initMixer();
  }
  catch(MixerDeviceException& e)
  {
    throw e;
  }
}

//----------------------------------------------------------------------
OSSMixer::OSSMixer(const string& device_name)
  throw(MixerDeviceException)
  : Mixer(device_name), modify_counter_(-1)
{
  try
  {
    initMixer();
  }
  catch(MixerDeviceException& e)
  {
    throw e;
  }
}


//----------------------------------------------------------------------
void OSSMixer::initMixer() throw(MixerDeviceException)
{
  if((file_descriptor_ = open(device_name_.c_str(),O_RDONLY | O_NONBLOCK)) != -1)
  {
    int device_mask;          // supported devices
    int stereo_mask;          // stereo devices
    int record_mask;          // devices which can be recorded from
    int record_source;        // devices which are being recorded from

    num_devices_           = SOUND_MIXER_NRDEVICES;
    char *device_labels_[] = SOUND_DEVICE_LABELS;
    ioctl(file_descriptor_, SOUND_MIXER_READ_DEVMASK, &device_mask);
    ioctl(file_descriptor_, SOUND_MIXER_READ_STEREODEVS, &stereo_mask);
    ioctl(file_descriptor_, SOUND_MIXER_READ_RECMASK, &record_mask);
    ioctl(file_descriptor_, SOUND_MIXER_READ_RECSRC,  &record_source);
    ioctl(file_descriptor_, SOUND_MIXER_READ_CAPS,    &capabilities_);
      
    devices_ = new OSSDevice*[num_devices_];

    int bit_position = 1;

    for(unsigned count=0; count<num_devices_; count++)
    {
      devices_[count] = new OSSDevice(device_mask   & bit_position,
				      stereo_mask   & bit_position,
				      record_mask   & bit_position,
				      record_source & bit_position,
				      bit_position,
				      device_labels_[count]);
      bit_position <<= 1;
    }
  }
  else
  {
    throw MixerDeviceException(device_name_);
  }

  doUpdate();
}
//----------------------------------------------------------------------
OSSMixer::~OSSMixer()
{
  close(file_descriptor_);

  for(unsigned count=0; count<num_devices_; count++)
  {
    delete devices_[count];
  }

  delete[](devices_);
}

//----------------------------------------------------------------------
void OSSMixer::writeChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  int dummy = devices_[channel_num]->getValue();

  if((ioctl(file_descriptor_, MIXER_WRITE(channel_num),
	    &dummy)) == -1)
  {
    std::cout << "OSSMixer::writeChannel, channel: " << devices_[channel_num]->getLabel() << ", value: " << devices_[channel_num]->getValue() << std::endl;
    std::cout << "   " << strerror(errno) << std::endl;
      
  }
}

//----------------------------------------------------------------------
void OSSMixer::muteChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->mute();
  writeChannel(channel_num);
}

//----------------------------------------------------------------------
void OSSMixer::unMuteChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->unMute();
  writeChannel(channel_num);
}


//----------------------------------------------------------------------
void OSSMixer::setChannelLeft(unsigned channel_num, unsigned value)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);
  
  devices_[channel_num]->setLeftValue(value);
  writeChannel(channel_num);
  doUpdate(channel_num);
}

//----------------------------------------------------------------------
void OSSMixer::setChannelRight(unsigned channel_num, unsigned value)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->setRightValue(value);
  writeChannel(channel_num);
  doUpdate(channel_num);
}

//----------------------------------------------------------------------
bool OSSMixer::toggleChannelRecord(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  if(!devices_[channel_num]->isRecordable())
    return false;

  bool val = devices_[channel_num]->toggleRecordSource();

  int record_sources = 0;

  for(unsigned count = 0; count < num_devices_; count++)
  {
    //       record_sources |= devices_[count]->isRecordSource() ? 
    // 	1 << devices_[count]->getMask() : 0;
    record_sources |= devices_[count]->isRecordSource() ? 
      devices_[count]->getMask() : 0;
  }

  ioctl(file_descriptor_, SOUND_MIXER_WRITE_RECSRC, &record_sources);

  return val;
}

//----------------------------------------------------------------------
bool OSSMixer::hasChanged()
{
  struct mixer_info mixer_info;
  ioctl(file_descriptor_, SOUND_MIXER_INFO, &mixer_info);
  
  if (mixer_info.modify_counter == modify_counter_)
  {
    return false;
  }
  else 
  {
    modify_counter_ = mixer_info.modify_counter;
    return true;
  }
}

//----------------------------------------------------------------------
void OSSMixer::doUpdate(unsigned num_channel)
{
  int dummy = 0;
  int record_source;

  ioctl(file_descriptor_, SOUND_MIXER_READ_RECSRC, &record_source);
  
  if(devices_[num_channel]->isSupported())
  {
    ioctl(file_descriptor_, MIXER_READ(num_channel), &dummy);
    devices_[num_channel]->setValue(dummy);
  }
  devices_[num_channel]->setRecordSource(record_source & 
					 devices_[num_channel]->getMask());
}

//----------------------------------------------------------------------
void OSSMixer::doUpdate()
{
  int dummy = 0;
  int record_source;

  ioctl(file_descriptor_, SOUND_MIXER_READ_RECSRC, &record_source);
  
  for(unsigned count = 0; count < num_devices_; count++)
  {
    if(devices_[count]->isSupported())
    {
      ioctl(file_descriptor_, MIXER_READ(count), &dummy);
      devices_[count]->setValue(dummy);
    }
    devices_[count]->setRecordSource(record_source & 
				     devices_[count]->getMask());
  }
}

//----------------------------------------------------------------------
bool OSSMixer::getCapture(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  int record_source;
  ioctl(file_descriptor_, SOUND_MIXER_READ_RECSRC, &record_source);
  devices_[channel_num]->setRecordSource(record_source & 
					 devices_[channel_num]->getMask());
  
  return devices_[channel_num]->isRecordSource();
}

//----------------------------------------------------------------------
bool OSSMixer::canCapture(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->isRecordable();
}


//----------------------------------------------------------------------
unsigned OSSMixer::getChannelLeft(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->getLeftValue();
}

//----------------------------------------------------------------------
unsigned OSSMixer::getChannelRight(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->getRightValue();
}


//----------------------------------------------------------------------
const string& OSSMixer::getChannelName(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->getLabel();
}

//----------------------------------------------------------------------
bool OSSMixer::isMuted(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->isMuted();
}

//----------------------------------------------------------------------
bool OSSMixer::isMono(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return !devices_[channel_num]->isStereo();
}

//----------------------------------------------------------------------
bool OSSMixer::isSupported(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->isSupported();
}

//----------------------------------------------------------------------
bool OSSMixer::isEnabled(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->isEnabled();
}

//----------------------------------------------------------------------
void OSSMixer::disableChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->disable();
}

//----------------------------------------------------------------------
void OSSMixer::enableChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->enable();
}


